@@ -398,11 +398,11 @@ class OAuthRemoteApp(object):
                              is provided, the data is passed as it, and
                              the `format` is ignored.
         :param token: an optional token to pass, if it is None, token will
-                      be generated be tokengetter.
+                      be generated by tokengetter.
         """
 
         headers = dict(headers or {})
-        if not token:
+        if token is None:
             token = self.get_request_token()
 
         client = self.make_client(token)
@@ -430,9 +430,9 @@ class OAuthRemoteApp(object):
             )
 
         if hasattr(self, 'pre_request'):
-            # this is desgined for some rubbish service like weibo
-            # since they don't follow the standards, we need to
-            # change the uri, headers, or body
+            # This is desgined for some rubbish service like weibo.
+            # Since they don't follow the standards, we need to
+            # change the uri, headers, or body.
             uri, headers, body = self.pre_request(uri, headers, body)
 
         resp, content = self.http_request(
@@ -619,25 +619,31 @@ class OAuthRemoteApp(object):
         """Handles a unknown authorization response."""
         return None
 
+    def handle_response(self):
+        """Handles authorization response smartly."""
+        if 'oauth_verifier' in request.args:
+            try:
+                data = self.handle_oauth1_response()
+            except OAuthException as e:
+                data = e
+        elif 'code' in request.args:
+            try:
+                data = self.handle_oauth2_response()
+            except OAuthException as e:
+                data = e
+        else:
+            data = self.handle_unknown_response()
+
+        # free request token
+        session.pop('%s_oauthtok' % self.name, None)
+        session.pop('%s_oauthredir' % self.name, None)
+        return data
+
     def authorized_handler(self, f):
+        """Handles an OAuth callback."""
         @wraps(f)
         def decorated(*args, **kwargs):
-            if 'oauth_verifier' in request.args:
-                try:
-                    data = self.handle_oauth1_response()
-                except OAuthException as e:
-                    data = e
-            elif 'code' in request.args:
-                try:
-                    data = self.handle_oauth2_response()
-                except OAuthException as e:
-                    data = e
-            else:
-                data = self.handle_unknown_response()
-
-            # free request token
-            session.pop('%s_oauthtok' % self.name, None)
-            session.pop('%s_oauthredir' % self.name, None)
+            data = self.handle_response()
             return f(*((data,) + args), **kwargs)
         return decorated
 
