@@ -49,8 +49,8 @@ class OAuth1Provider(object):
 
         @app.route('/api/user')
         @oauth.require_oauth('email', 'username')
-        def user(oauth):
-            return jsonify(oauth.user)
+        def user():
+            return jsonify(request.oauth.user)
     """
 
     def __init__(self, app=None):
@@ -491,6 +491,9 @@ class OAuth1Provider(object):
                 for func in self._before_request_funcs:
                     func()
 
+                if hasattr(request, 'oauth') and request.oauth:
+                    return f(*args, **kwargs)
+
                 server = self.server
                 uri, http_method, body, headers = extract_params()
                 valid, req = server.validate_protected_resource_request(
@@ -501,10 +504,11 @@ class OAuth1Provider(object):
                     valid, req = func(valid, req)
 
                 if not valid:
-                    return abort(403)
+                    return abort(401)
                 # alias user for convenience
                 req.user = req.access_token.user
-                return f(*((req,) + args), **kwargs)
+                request.oauth = req
+                return f(*args, **kwargs)
             return decorated
         return wrapper
 
