@@ -72,6 +72,7 @@ class OAuth2Provider(object):
     def __init__(self, app=None):
         self._before_request_funcs = []
         self._after_request_funcs = []
+        self._invalid_response = None
         if app:
             self.init_app(app)
 
@@ -199,6 +200,22 @@ class OAuth2Provider(object):
         self._after_request_funcs.append(f)
         return f
 
+    def invalid_response(self, f):
+        """Register a function for responsing with invalid request.
+
+        When an invalid request proceeds to :meth:`require_oauth`, we can
+        handle the request with the registered function. The function
+        accepts one parameter, which is an oauthlib Request object::
+
+            @oauth.invalid_response
+            def invalid_require_oauth(req):
+                return jsonify(message=req.error_message), 401
+
+        If no function is registered, it will return with ``abort(401)``.
+        """
+        self._invalid_response = f
+        return f
+
     def clientgetter(self, f):
         """Register a function as the client getter.
 
@@ -355,18 +372,28 @@ class OAuth2Provider(object):
                     scopes, credentials = ret
                     kwargs['scopes'] = scopes
                     kwargs.update(credentials)
-                    return f(*args, **kwargs)
                 except oauth2.FatalClientError as e:
                     log.debug('Fatal client error %r', e)
                     return redirect(e.in_uri(self.error_uri))
 
-            if request.method == 'POST':
+            else:
                 redirect_uri = request.values.get('redirect_uri', None)
-                if not f(*args, **kwargs):
-                    # denied by user
-                    e = oauth2.AccessDeniedError()
-                    return redirect(e.in_uri(redirect_uri))
-                return self.confirm_authorization_request()
+
+            try:
+                rv = f(*args, **kwargs)
+            except oauth2.FatalClientError as e:
+                log.debug('Fatal client error %r', e)
+                return redirect(e.in_uri(self.error_uri))
+
+            if not isinstance(rv, bool):
+                # if is a response or redirect
+                return rv
+
+            if not rv:
+                # denied by user
+                e = oauth2.AccessDeniedError()
+                return redirect(e.in_uri(redirect_uri))
+            return self.confirm_authorization_request()
         return decorated
 
     def confirm_authorization_request(self):
@@ -442,6 +469,8 @@ class OAuth2Provider(object):
                     valid, req = func(valid, req)
 
                 if not valid:
+                    if self._invalid_response:
+                        return self._invalid_response(req)
                     return abort(401)
                 request.oauth = req
                 return f(*args, **kwargs)
@@ -567,7 +596,8 @@ class OAuth2RequestValidator(RequestValidator):
         log.debug('Compare redirect uri for grant %r and %r.',
                   grant.redirect_uri, redirect_uri)
 
-        if os.environ.get('DEBUG') and redirect_uri is None:
+        testing = 'OAUTHLIB_INSECURE_TRANSPORT' in os.environ
+        if testing and redirect_uri is None:
             # For testing
             return True
 
@@ -660,17 +690,23 @@ class OAuth2RequestValidator(RequestValidator):
         log.debug('Validate bearer token %r', token)
         tok = self._tokengetter(access_token=token)
         if not tok:
-            log.debug('Bearer token not found.')
+            msg = 'Bearer token not found.'
+            request.error_message = msg
+            log.debug(msg)
             return False
 
         # validate expires
         if datetime.datetime.utcnow() > tok.expires:
-            log.debug('Bearer token is expired.')
+            msg = 'Bearer token is expired.'
+            request.error_message = msg
+            log.debug(msg)
             return False
 
         # validate scopes
         if not set(tok.scopes).issuperset(set(scopes)):
-            log.debug('Bearer token scope not valid.')
+            msg = 'Bearer token scope not valid.'
+            request.error_message = msg
+            log.debug(msg)
             return False
 
         request.access_token = tok
